@@ -118,6 +118,20 @@ def getchildren(segm, word):
     idx2 = np.argwhere(idx).reshape(-1).astype(int)
     return [segm[x] for x in idx2]
 
+def parse_align_file(align_file):
+    labels = []
+    instants = []
+    
+    with open(align_file, 'r', encoding='Latin 1') as f:    
+        for curr_line in f.readlines()   :
+            curr_split = curr_line.split(' ')
+            start, dur, phone = curr_split[-3:]
+            labels.append(phone.replace("\n", ""))            
+            instants.append([float(start), 
+                             float(start) + float(dur)])
+            
+    return [labels], [np.array(instants)]
+
 def read_xml_segmentation_files(fileName):
               
     start = []
