@@ -8,13 +8,21 @@ Created on Thu Jun 25 07:54:59 2020
 
 from os import path
 from .speechaudio import SpeechAudio
-import soundfile
+
 import numpy as np
 
 def file2speech(fileName, chn=1, sro=None, start=0, end=-1):
     
     if path.isfile(fileName):
-        y, sr = soundfile.read(fileName)
+        _, ext = path.splitext(fileName)
+        if ext == ".mp3":
+            import librosa
+            import warnings
+            warnings.filterwarnings('ignore')
+            y, sr = librosa.load(fileName)
+        else:
+            import soundfile 
+            y, sr = soundfile.read(fileName)
         if len(y.shape) == 2:
             idx = np.argmax(y.shape)
             if idx == 0:
@@ -35,5 +43,5 @@ def file2speech(fileName, chn=1, sro=None, start=0, end=-1):
             if sr != sro:
                 SpeechObject.speechresample(sro)
     else:
-        raise ValueError("File does not exist")    
+        raise ValueError("File " + fileName + " does not exist")    
     return SpeechObject
\ No newline at end of file
