@@ -12,6 +12,7 @@ import statsmodels.api as sm
 import scipy.interpolate as scintp
 import warnings
 from .plottools import pitch_plot, contour_plot
+from .splinetools import (check_list, move_spline, target_spline, translate_spline)
 
 class Pitch:
     values = None
@@ -66,13 +67,13 @@ class Pitch:
         self.values = y_out
         self.time_points = time_out
 
-    def getcontour(self):
+    def getcontour(self, len_win=0.3):
 
         f0 = self.values
         tf0 = self.time_points
 
         srF0 = 1/(tf0[2]-tf0[1])
-        nwin = int(0.3 * srF0)
+        nwin = int(len_win * srF0)
         if np.mod(nwin, 2) > 0:
             nwin = nwin + 1
         novlp = int(nwin * 1 / 2)
@@ -127,7 +128,7 @@ class Pitch:
             spl_v = []
             
         return pitch_plot(self.time_points, self.values, self.contour.values,
-                          spl_tp, spl_v)
+                          spl_tp, spl_v, self.parent)
 
     def shift(self, factor):
         self.values = self.values * factor
@@ -225,17 +226,21 @@ class Contour(Pitch):
             key, value = args[k:k+2]
             setattr(self, key, value)
 
-    def getspline(self):
+    def getspline(self, len_win=0.15):
 
         tf0 = self.time_points
         f0 = self.values
-
-        srOut = 1/0.15
+        
+        srOut = 1/len_win
 
         srIn = 1/(tf0[2]-tf0[1])
         D = int(srIn / srOut)
-        newTime = tf0[::D]
-        newPitch = f0[::D]
+        if D > 1:
+            newTime = tf0[::D]
+            newPitch = f0[::D]
+        else:
+            newTime = tf0
+            newPitch = f0
         spl = scintp.splrep(newTime, newPitch)
         splFreq = scintp.splev(tf0, spl)
         (t, c, k) = spl
@@ -250,54 +255,32 @@ class Contour(Pitch):
         self.spline = splineObj
 
         return splineObj
-
-    def changespline(self, pts, factor):
-
+    
+    def check_spline(self):
         if self.spline is None:
             self.getspline()
 
-        spline = self.spline
-        coeff = spline.coeff
-        knots = spline.knots
+    def changespline(self, pts, factor, target=False):
 
-        if pts.shape == ():
-            pts = [pts]
+        self.check_spline()
+        pts = check_list(pts)
+        if target:  
+            target_spline(self.spline, pts, factor) 
+        else:
+            translate_spline(self.spline, pts, factor)
 
-        idx = np.array([(np.abs(knots - x)).argmin() - 2 for x in pts])
-        coeff[idx] = coeff[idx] * factor
-
-        spline.coeff = coeff
-        spline.interpolate()
-
-        self.spline = spline
-        self.parent.values = (self.parent.values - self.parent.contour.values +
-                              spline.values)
-
-
-        self.values = spline.values
+        if hasattr(self.parent, "values"):
+            self.parent.values = (self.parent.values - self.parent.contour.values +
+                              self.spline.values)
+        self.values = self.spline.values
 
     def shiftspline(self, pts, factor):
 
-        if self.spline is None:
-            self.getspline()
+        self.check_spline()
+        pts = check_list(pts)
+        move_spline(self.spline, pts, factor)
 
-        spline = self.spline
-        knots = spline.knots
-
-        if pts.shape == ():
-            pts = [pts]
-
-        idx = np.array([(np.abs(knots - x)).argmin() for x in pts])
-        knots[idx] = knots[idx] + factor
-
-        idxSort = np.argsort(knots)
-
-        spline.knots = knots[idxSort]
-        spline.coeff = spline.coeff[idxSort]
-        spline.interpolate()
-
-        self.spline = spline
-        self.values = spline.values
+        self.values = self.spline.values
 
     def getlocalmax(self):
 
@@ -413,10 +396,11 @@ class Contour(Pitch):
         return self.local_peak_dynamic
 
     def db2lin(self):
-
         return  (10**(self.values / 10)) * 2e-5
+    def lin2db(self):
+        return  10 * np.log10(self.values / 2e-5)
 
-class Spline(Pitch):
+class Spline(Contour):
     parent = None
     coeff = None
     knots = None
@@ -445,8 +429,10 @@ class Intensity(Contour):
             key, value = args[k:k+2]
             setattr(self, key, value)
 
-    def interpolate(self, SpObj):
+    def interpolate(self, SpObj=None):
 
+        if SpObj is None:
+            SpObj = self.parent
         y = SpObj.signal
         sr = SpObj.sampling_frequency
 
@@ -469,14 +455,14 @@ class Intensity(Contour):
 
         return intOut
 
-    def getcontour(self):
+    def getcontour(self, len_win=0.15):
 
         x = self.values
         tVec = self.time_points
 
         sr = 1/(tVec[2]-tVec[1])
 
-        winOrd = int(0.15 * sr)
+        winOrd = int(len_win * sr)
         xOut = sm.nonparametric.lowess(x, np.arange(len(x)),
                                        frac=winOrd/len(x),
                                        return_sorted=False)
@@ -500,7 +486,8 @@ class Intensity(Contour):
         yI = self.db2lin()
 
         return contour_plot(tVecSignal, yS, 
-                            self.time_points, yI * np.max(yS) / np.max(yI))
+                            self.time_points, yI * np.max(yS) / np.max(yI),
+                            parent=self.parent)
 
     def changespan(self, factor):
 
