@@ -25,7 +25,8 @@ import copy
 from .ola import *
 from .praattools import *
 from .transformtools import (transform_formant, transform_formant_from_dist,
-                             sparse_transform)
+                             sparse_transform, get_pros_modif_factor_from_dist,
+                             transform_pitch)
 
 class SpeechAudio:
 
@@ -77,7 +78,7 @@ class SpeechAudio:
             nfft = 2**nextpow2(self.sampling_frequency)
         if novlp is None:
             novlp = int(nwin * 3 / 4)
-        
+
         x = self.signal
         sr = self.sampling_frequency
         nhop = nwin - novlp
@@ -95,7 +96,7 @@ class SpeechAudio:
         xwin = xbuff * w
         # xwin[xwin==0] = 1e-22
         # xEnv = np.zeros((nfft, nFrame))
-        
+
         Sxx = np.fft.fft(xwin, n=nfft, axis=0)[:L,:]
         # fxx, txx, Sxx = scipy.signal.spectrogram(self.signal, self.sampling_frequency,
         #                                 win, winLength, novlp, nfft)
@@ -223,31 +224,30 @@ class SpeechAudio:
         sOut = praat_change_shimmer(sound, factor, f0min=70, f0max=300)
         sound2sp(sOut, self)
 
-    def getintensity(self):
+    def getintensity(self, len_win=0.03):
 
         y = self.signal.squeeze()
         sr = self.sampling_frequency
+        tsig = np.arange(0, len(y)) / sr
 
-        nwin = int(0.04 * sr)
+        nwin = int(len_win * sr)
         if np.mod(nwin, 2) > 0:
             nwin = nwin + 1
 
-        novlp, nfft, nhop = ola_param(nwin, novlp=None)
-        w = scipy.signal.hann(nwin)
-        yOut = []
+        energy = []
+        s = 0
+        for k, smpl in enumerate(y):
+            s += smpl**2
+            if (k + 1) % nwin == 0:
+                energy.append(s)
+                s = 0
+            elif k == len(y) - 1:
+                energy.append(s)
 
-        k = 0
-        isC = True
-        while isC:
-            xTmp, idx, isC = check_index(y, k, nwin, nhop)
-            xTmp = (np.sum(xTmp)**2) / np.sum(w[:len(idx)])
-            yOut.append(xTmp)
-            k += 1
-
-        yOut = np.array(yOut) / 1.5
-        yOut = 10 * np.log10(yOut / 2e-5)
-        tVec = np.arange(len(yOut)) / (sr / nhop) + 0.5 * (nwin / sr)
-        objOut = Intensity('values', yOut,
+        yOut_lin = np.array(energy) / np.sqrt(nwin)
+        yOut_dB = 10 * np.log10(yOut_lin / 2e-5)
+        tVec = tsig[::nwin] + len_win / 2
+        objOut = Intensity('values', yOut_dB,
                            'time_points', tVec,
                            'parent', self)
 
@@ -257,20 +257,17 @@ class SpeechAudio:
     def changeintensity(self, intTarget):
 
         if self.intensity is None:
-            intIn = self.getIntensity()
+            intIn = self.getintensity()
         else:
             intIn = self.intensity
         intIn = intIn.interpolate(self)
         intTarget = intTarget.interpolate(self)
 
         y = self.signal
-        ydetrend = y / intIn.values
-        ychange = ydetrend * intTarget.values
+        ydetrend = y / intIn.db2lin()
+        ychange = ydetrend * intTarget.db2lin()
 
-        SpOut = self.copy()
-        SpOut.signal = ychange
-
-        return SpOut
+        self.signal = ychange
 
     def changetilt(self, factor, durWin=0.04, f0min=70, f0max=300):
 
@@ -654,16 +651,11 @@ class SpeechAudio:
         w = np.matlib.repmat(w.reshape(-1,1), 1, nFrame)
         xwin = xbuff * w
         xwin[xwin==0] = 1e-22
-        xEnv = np.zeros((nfft, nFrame))
+        Sxx = np.zeros((L, nFrame))
 
         for k in range(nFrame):
             order = get_order_ceps(self.pitch.values[k], sr)
-            if toldB is not None:
-                xEnv[:,k] = true_envelope(xwin[:,k], sr, toldB, order)
-            else:
-                logx = np.log(abs(np.fft.fft(xwin[:,k])))
-                xEnv[:,k] = cepstral_filtering(logx, nwin, order)
-        Sxx = np.exp(xEnv[:L, :])
+            Sxx[:,k] = cepstrum(xwin[:,k], sr, order, toldB)
 
         outputEnvelope = TimeFreq('parent', self,
                                   'time_frequency_matrix', Sxx,
@@ -676,32 +668,32 @@ class SpeechAudio:
         self.spectral_envelope = outputEnvelope
 
         return Sxx, fxx, txx
-    
+
     def mfcc(self, nwin=512, novlp=None, nfft=1024, nb_filter=24, nb_coeff=12,
               preemph=[1, -0.95], win='hamming', replace=False):
         """ computes the MFCC coefficients """
-        
+
         if novlp is None:
             novlp = int(nwin * 3 / 4)
         if nfft is None:
             nfft = nwin
-            
+
         Sp = self.copy()
-            
+
         if preemph is not None:
             Sp.signal = scipy.signal.lfilter([1], preemph, Sp.signal)
         Sxx, fxx, txx = Sp.computespectrogram(win=win, nwin=nwin, novlp=novlp,
                                               nfft=nfft)
         filters, mels, mel_freqs = mel_filter_bank(sr=self.sampling_frequency,
-                                                   fmin=0, fmax=None, 
-                                                   nb_filter=nb_filter, 
+                                                   fmin=0, fmax=None,
+                                                   nb_filter=nb_filter,
                                                    nfft=nfft)
-        
+
         Pxx = np.abs(Sxx)**2
         energy = np.sum(Pxx, axis=0).reshape(1, -1)
         Pxx_filtered = filters @ Pxx
         Pxx_log = 10 * np.log10(Pxx_filtered)
-        
+
         # dct_basis = dct(nb_coeff + 1, filter_len=nb_filter)[1:,:]
         dct_basis = dct(nb_coeff + 1, filter_len=nb_filter)
         static_coeff = dct_basis @ Pxx_log
@@ -711,13 +703,13 @@ class SpeechAudio:
         dt = delta(static_coeff)
         ddt = delta(dt)
         mfcc_coeff = np.vstack((static_coeff, dt, ddt))
-        
+
         mfcc = MFCC('coeff', mfcc_coeff, 'time_vector', txx,
                     'mel_vector', mels, 'frequency_vector', mel_freqs,
                     'power_log', Pxx_log, 'nb_coeff', nb_coeff, 'c0', c0,
                     'energy', energy, 'parent', self)
         mfcc.mfcc2melspectro()
-        
+
         self.mfcc = mfcc
 
     def addformants(self, tf_add, nwin=512, novlp=None, nfft=None, nargout=1):
@@ -776,6 +768,8 @@ class SpeechAudio:
                 labels, instants = read_seg_segmentation_files(fileName)
             else:
                 labels, instants = read_seg_segmentation_files_2(fileName, utt_id)
+        elif fileName.lower().find('.ctm') > 0:
+            labels, instants = parse_align_file(fileName)           
         else:
             raise FormatError('Unknown file format')
 
@@ -783,29 +777,22 @@ class SpeechAudio:
         self.phonetic_instants = instants
 
         return labels, instants
-    
-    def sparse_transform(self, W_nominal, W_mask, nwin=240, novlp=None):
 
+    def sparse_transform(self, W_source, W_target, isFlat=True, isNorm=True):
+
+        nb_feat, nb_atoms = W_source.shape
+        nwin = (nb_feat - 1) * 2
+        novlp = int(nwin * 3 / 4)
         novlp, nfft, nhop = ola_param(nwin, novlp)
         x2, x_out = init_ola(self.signal, nwin, nhop)
-        
-        Spx2 = SpeechAudio("signal", x2, 
+        xx = buffer(x2, nwin, nhop)
+        Spx2 = SpeechAudio("signal", x2,
                            "sampling_frequency", self.sampling_frequency)
         Sxx, fxx, txx = Spx2.computecepstrumenvelope(nwin, novlp)
-        Sxx_target = sparse_transform(Sxx, W_nominal, W_mask)
-        k = 0
-        isC = True
-        while isC:
-            xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
-            h_orig = np.abs(Sxx[:, k])
-            h_new = np.abs(Sxx_target[:, k])
-            x_out = change_frame(fft_xwin, h_new, h_orig,
-                                         nfft, idx, x_out, xwin=xwin, isNorm=False)
-    
-            k, isC = check_frame(k, isC, maxFrame=Sxx.shape[1])
-        x_out = x_out[nhop:-nhop] / 1.5
-        x_out[np.isnan(x_out)] = 0
-        self.signal = x_out
+        Sxx_target = sparse_transform(Sxx, W_source, W_target)
+
+        self.signal = change_cepstrum(x2, Spx2, x_out, nwin, nhop, nfft, txx, isFlat, isNorm,
+                    Sxx, Sxx_target)
 
     def cepstraltransform(self, transOper, nwin=1024,
                           novlp=None, sr=8000, freq_out=None,
@@ -829,15 +816,15 @@ class SpeechAudio:
             nb_filter = len(transOper.mel_vector) - 2
             S_tmp = transOper.mfcc2melspectro(freq_oo)
             Sxx_trans = S_tmp.time_frequency_matrix
-            
-            Spx2.mfcc(nwin=nwin, novlp=novlp, 
+
+            Spx2.mfcc(nwin=nwin, novlp=novlp,
                       nb_filter=nb_filter, nb_coeff=nb_coeff)
             S_tmp_2 = Spx2.mfcc.mfcc2melspectro(freq_oo)
             Sxx_in = S_tmp_2.time_frequency_matrix
-            
+
             fxx = S_tmp_2.frequency_vector
             txx = S_tmp_2.time_vector
-            
+
         if method.lower() == 'dnn':
             """ DNN-based modification is in progress """
             pass
@@ -881,7 +868,7 @@ class SpeechAudio:
                 Sxx_trans = a.time_frequency_matrix
         k = 0
         isC = True
-        
+
         while isC:
             xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
 
@@ -946,7 +933,6 @@ class SpeechAudio:
         snd = sp2sound(self)
         time_vec = np.arange(0, len(self.signal) + nhop, nhop) / sr
         form_out, bw_out, txx = praat_get_formants(snd, nhop, nwin)
-
         for k in range(4):
             fTmp = np.squeeze(form_out[k, :])
             fTmp = fill_nan_and_zero(fTmp)
@@ -981,6 +967,7 @@ class SpeechAudio:
                                  'isForm', True)
 
         formantObject.interpolate(time_vec)
+        formantObject.getcontour()
         self.formants = formantObject
 
         return formantObject
@@ -1043,14 +1030,29 @@ class SpeechAudio:
 
         self.signal = x_out
         self.formants = formTarget
-       
+        
+    def change_prosody(self, dil_fact=1, trans_fact=1, dur_fact=1, dist=None):
+
+        if self.pitch is None:
+            self.getpitch()
+        if dist is not None:
+            trans_fact, dil_fact, dur_fact = get_pros_modif_factor_from_dist(dist)
+            
+        if dil_fact != 1 and trans_fact != 1:
+            pitch_output = transform_pitch(self.pitch, trans_fact, dil_fact)
+            self.changecontour(pitch_output)
+
+        if dur_fact != 1:
+            pts = [0, len(self.signal)/ self.sampling_frequency]
+            factor = np.ones(3) * dur_fact
+            self.changeduration(pts, factor)
+
     def frequency_warping(self, dil_fact, trans_fact, dist=None, ord_filt=None,
-                      nwin=512, novlp=None):
+                      nwin=512, novlp=None, isNorm=True, isFlat=True):
 
         novlp, nfft, nhop = ola_param(nwin, novlp)
         x2, x_out = init_ola(self.signal, nwin, nhop)
-        
-        Spx2 = SpeechAudio('signal', x2, 
+        Spx2 = SpeechAudio('signal', x2,
                            'sampling_frequency', self.sampling_frequency)
         Sxx, fxx, txx = Spx2.computecepstrumenvelope(nwin, novlp)
         form_in = Spx2.formantestimate(nwin, novlp, ord_filt=ord_filt)
@@ -1061,53 +1063,71 @@ class SpeechAudio:
         else:
             form_output = transform_formant_from_dist(form_in, dist)
         form_output.interpolate(txx)
-        
-        k = 0
-        isC = True
-        while isC:
-            xwin, fft_xwin, isC, idx = spec_frame(x2, k, nwin, nhop, nfft)
-            h_orig = np.abs(Sxx[:, k])
-            if (form_in.frequency[:,k] != 0).all():
-                # new_freq = ceps2peaks(h_orig, form_in.frequency[:,k], fxx)
-                new_freq = form_in.frequency[:,k]
-                h_new = dfw(h_orig, fxx, np.sort(form_output.frequency[:,k]), new_freq)
-            else:
-                h_new = 1
-    
-            x_out = change_frame(fft_xwin, h_new, h_orig,
-                                         nfft, idx, x_out, xwin=xwin,
-                                         isChange=(form_in.frequency[:,k] != 0).all())
-    
-            k, isC = check_frame(k, isC, maxFrame=Sxx.shape[1])
-        x_out = x_out[nhop:-nhop] / 1.5
-        x_out[np.isnan(x_out)] = 0
-        self.signal = x_out
+
+        nb_frame = Sxx.shape[1]
+        xx, yy = np.meshgrid(np.arange(0, nb_frame), fxx)
+        h_new = dfw_matrix(np.abs(Sxx), fxx,
+                           np.sort(form_output.frequency, axis=0),
+                           form_in.frequency)
+
+        self.signal = change_cepstrum(x2, Spx2, x_out, nwin, nhop, nfft, txx,
+                                      isFlat, isNorm, Sxx, h_new)
         self.formantestimate(nwin, novlp, ord_filt=None)
         self.formants.interpolate(txx)
-        
+
     def time_warping(self, inst_target, tiers=0):
         """ dynamic time warping of phonemes to match the specified target """
         if inst_target.shape[0] != self.phonetic_instants[tiers].shape[0]:
             raise ValueError('The number of phonemes does not match')
-            
+
         len_input = len(self.signal) / self.sampling_frequency
-        inst_input = self.phonetic_instants[0]                     
+        inst_input = self.phonetic_instants[0]
         if inst_input[-1,1] < len_input:
-            inst_input[-1,1] = len_input   
-     
+            inst_input[-1,1] = len_input
+
         dur_input =  inst_input[:,1]- inst_input[:,0]
         dur_target = inst_target[:, 1] - inst_target[:, 0]
-        
+
         factor = dur_target / dur_input
         factor[np.isnan(factor)] = 0
         factor[np.isinf(factor)] = 0
-        
+
         pts = inst_input[:-1,1]
-        self.changeduration(pts, factor, isSeg=True) 
+        self.changeduration(pts, factor, isSeg=True)
+
+    def remove_silence(self, side="both"):
+        y = self.signal
+        sr = self.sampling_frequency
+        lbl = self.phonetic_labels[0]
+        inst = self.phonetic_instants[0]
+
+        if lbl[0] == "." and side in ["both", "l"]:
+            delay = inst[1,0]
+            idx_st = int(delay * sr)
+            y = y[idx_st:]
+            lbl = lbl[1:]
+            inst = inst[1:, :] - delay
+            if len(self.phonetic_labels) > 1:
+                if self.phonetic_labels[1][0] ==  "[silence]":
+                    self.phonetic_labels[1] = self.phonetic_labels[1][1:]
+                    self.phonetic_instants[1] = self.phonetic_instants[1][1:, :] - delay
+        if lbl[-1] == "." and side in ["both", "r"]:
+            idx_st = int(inst[-1,0] * sr)
+            y = y[:idx_st]
+            lbl = lbl[:-1]
+            inst = inst[:-1,:]
+            if len(self.phonetic_labels) > 1:
+                if self.phonetic_labels[1][-1] ==  "[silence]":
+                    self.phonetic_labels[1] = self.phonetic_labels[1][:-1]
+                    self.phonetic_instants[1] = self.phonetic_instants[1][:-1, :]
+
+        self.phonetic_labels[0] = lbl
+        self.phonetic_instants[0] = inst
+        self.signal = y
 
     def savespeech(self, fileName, norm=None):
         """ saves speech into an audio file """
-        
+
         if norm is None:
             norm = 'notnormalized'
         if norm is type(str):
@@ -1162,7 +1182,7 @@ class SpeechAudio:
                 phonSegLbl.append(lblTmp)
                 phonSegInst.append(momTmp)
 
-            idx_sig = [x for x in range(int(inst_tmp[0] * sr), 
+            idx_sig = [x for x in range(int(inst_tmp[0] * sr),
                                         int(inst_tmp[1] * sr)+1) if x < len(self.signal)]
             phon_tmp.signal = x[np.unique(idx_sig)]
 
@@ -1229,7 +1249,7 @@ class SpeechAudio:
         else:
             print('Warning : The output sampling frequency '
                   'is the same as the input. Nothing''s done!')
-            
+
     def computeVQ(self, nwin=1024, novlp=None, nfft=None, meth="rms"):
 
         if self.voiced is None or self.unvoiced is None:
@@ -1237,19 +1257,19 @@ class SpeechAudio:
 
         sr = self.sampling_frequency;
         tvec = np.arange(0, len(self.signal)) / sr
-        
+
         if novlp is None:
             novlp = int(nwin * 3 / 4)
-        if nfft is None: 
+        if nfft is None:
             nfft = 2**nextpow2(sr)
-        
+
         V = self.voiced
         U = self.unvoiced
-        
+
         if meth.lower() == "spectro":
             Sxx_V, fxx, txx = V.computespectrogram(nwin, novlp=novlp, nfft=nfft)
             Sxx_U, _, _ = U.computespectrogram(nwin, novlp=novlp, nfft=nfft)
-            
+
             Den = np.sum(abs(Sxx_U)**2, 0) + np.sum(abs(Sxx_V)**2, 0)
             grossVQ = np.sum(abs(Sxx_V)**2, 0) / Den
             grossHNR = 20*np.log10(abs((np.sum(abs(Sxx_V)**2, 0))/
@@ -1261,15 +1281,15 @@ class SpeechAudio:
                 nFrame = Vbuff.shape[1]
                 txx = np.arange(0,len(self.signal) - 1, nhop) / sr
                 txx = txx[:nFrame] + nwin / sr / 2
-        
+
                 grossVQ = np.zeros(nFrame)
                 grossHNR = np.zeros(nFrame)
                 w = scipy.signal.hann(nwin)
-        
+
                 for k in range(nFrame):
-                    xpt = Vbuff[:,k] * w 
+                    xpt = Vbuff[:,k] * w
                     xpt = xpt - np.mean(xpt)
-                    xnt = Ubuff[:,k] * w 
+                    xnt = Ubuff[:,k] * w
                     xnt = xnt - np.mean(xnt)
                     grossVQ[k] = ((np.linalg.norm(xpt,2)**2)/
                                   (np.linalg.norm(xpt,2)**2+np.linalg.norm(xnt,2)**2))**2
@@ -1281,13 +1301,13 @@ class SpeechAudio:
         VQ[VQ < 0] = 0
         VQ[VQ > 100] = 100
         HNR = lininterp1(txx, grossHNR, tvec)
-        
+
         self.voicing_quotient = VQ
         self.hnr = HNR
-        
+
         if self.time_vector is None:
             self.time_vector = tvec
-            
+
         return VQ, HNR
 class SpeechSegment(SpeechAudio):
 
