@@ -140,6 +140,7 @@ class Formants:
     damping = None
     amplitude = None
     isForm = None
+    trajectory = None
 
     time_vector = None
     parent = None
@@ -175,7 +176,27 @@ class Formants:
         'time_vector', self.time_vector)
 
         return zeroObj
-
+    
+    def getcontour(self):
+        
+        nb_form = self.frequency.shape[0]
+        self.trajectory = []
+        for k in range(nb_form):
+            trj_tmp = Contour("time_points", self.time_vector, 
+                              "values", self.frequency[k,:], 
+                              "parent", self)
+            self.trajectory.append(trj_tmp)
+            
+    def update_trajectory(self):
+        
+        if self.trajectory is not None:
+            self.interpolate(self.trajectory[0].time_points)
+            self.frequency = []
+            for trj in self.trajectory:
+                self.frequency.append(trj.values)
+                self.time_vector = trj.time_points
+            self.frequency = np.array(self.frequency)
+    
     def transform(self, sr, *args):
 
         freq = self.frequency
@@ -270,31 +291,12 @@ class Formants:
         return obj_out
 
     def interpolate(self, txx):
-
-        tin = self.time_vector
-        freq = self.frequency
-        damp = self.damping
-        bw = self.bandwidth
-        ampl = self.amplitude
-
-        f1 = scintp.interp1d(tin, freq, kind='linear', axis=1,
-                             bounds_error=False, fill_value=0)
-        freq_out = f1(txx)
-        f1 = scintp.interp1d(tin, damp, kind='linear', axis=1,
-                             bounds_error=False, fill_value=0)
-        damp_out = f1(txx)
-
-        f1 = scintp.interp1d(tin, bw, kind='linear', axis=1,
-                             bounds_error=False, fill_value=0)
-        bw_out = f1(txx)
-        f1 = scintp.interp1d(tin, ampl, kind='linear', axis=1,
-                             bounds_error=False, fill_value=0)
-        ampl_out = f1(txx)
-
-        self.frequency = freq_out
-        self.damping = damp_out
-        self.bandwidth = bw_out
-        self.amplitude = ampl_out
+        """ Interpolation of formant trajectory along the time axis """
+        for attr in ["frequency", "damping", "bandwidth", "amplitude"]:
+            f1 = scintp.interp1d(self.time_vector, getattr(self, attr), 
+                                 kind='linear', axis=1, 
+                                 bounds_error=False, fill_value=0)
+            setattr(self, attr, f1(txx))       
         self.time_vector = txx
         self.extrap()
         
