@@ -64,6 +64,12 @@ def get_distribution_from_file(dist_file):
         
     return a
 
+def get_pros_modif_factor_from_dist(dist):
+    fact_med = random_smpl(dist, 'med_pitch', lim=[0.5,2])
+    fact_span = random_smpl(dist, 'span_pitch', lim=[0.25,3])
+    fact_dur = random_smpl(dist, 'duration', lim=[1/1.8,1.8])
+    return fact_med, fact_span, fact_dur
+
 def parse_align_file(align_file):
     f = open(align_file, 'r', encoding='Latin 1')
     lines = f.readlines()
@@ -161,7 +167,7 @@ def transform_sparse_audio(Sp, W_source, W_target, output_audio):
     nb_feat, nb_atoms = W_source.shape
     nwin = (nb_feat - 1) * 2
     novlp = int(nwin * 3 / 4)
-    Sp.sparse_transform(W_source, W_target, nwin=nwin, novlp=novlp)
+    Sp.sparse_transform(W_source, W_target)
     Sp.savespeech(output_audio, "normalized")    
     
     return len(Sp.signal) / Sp.sampling_frequency
@@ -199,4 +205,19 @@ def transform_formant_from_dist(Fr, dist, rndmize=True):
         Cont.values *= fact_med 
         Fr_out.frequency[k,:] = Cont.values
     
-    return Fr_out
\ No newline at end of file
+    return Fr_out
+
+def transform_pitch(pitch, fact_med, fact_span):
+    
+    pitch_out = pitch.copy()    
+    pitch_out.getcontour()
+    if fact_span != 1:
+        try:
+            pitch_out.changespan(fact_span)
+        except:
+            pitch_out = pitch.copy()
+            pitch_out.getcontour()
+    if fact_med != 1:
+        pitch_out.values *= fact_med
+     
+    return pitch_out
\ No newline at end of file
