@@ -16,7 +16,7 @@ from ._set import struct, timestruct
 from .vtnetwork import VtNetwork
 from .areafunction import AreaFunction
 from .acoustics import AcousticsTime, AcousticsFreq
-from .plottools import plot_tf
+#from .plottools import plot_tf
 
 
 class VtWaveguide(VtNetwork):
@@ -567,20 +567,7 @@ class VtWaveguide(VtNetwork):
         Hnest = self.computetransferfunction(param)
 
         return akmat, lkmat, fnest, Hnest, err_form
-
-    def plottransferfunction(self, sl=None, hf=None):
-        """ plot the transfer function of the waveguide"""
-
-        if self.transfer_function is None:
-            print('Warning: No transfer function computed for this waveguide!'
-                  ' Please compute the transfer function first')
-            return None
-
-        tf = self.transfer_function
-        freq = self.freq
-
-
-        return plot_tf(freq, tf, sl, hf)
+    
     
     def check_instance(self, class_type):
         """ check instance """
